%------------------------------------------------- Comparing two CDFs
function CDF_diff=CDFcompare(CDF1,CDF2, display, qualityCurve)
% Checks and possibly adjusts the CDFs
% Note that the numbers of data MUST be the same
% and also the min and the max on the abscisse axis
CDF1x=CDF1(1,:);
CDF1y=CDF1(2,:);
CDF2x=CDF2(1,:);
CDF2y=CDF2(2,:);

if CDF1x ~=CDF2x
    error('\n CDF1x and CDF2x must be the same');
end

% Computes the weighted integral according to a quality function
% The idea is to assign more "weight" to the best results
nbBins=length(CDF1x);
if display>0 fprintf('\n %i bins, [%f %f]',n,CDF1x(1),CDF1x(n)); end

CDF_diff=0;
for n=1:nbBins
    u=(n-1)/(nbBins-1);
    
    switch qualityCurve
        case 1 % Linear
            weight=(nbBins-n)/(nbBins-1);
            
        case 2 %  Parabolic
            weight=1-u^2;
            
        case 3 %  Exponential
            weight=exp(-3*u);
            
        otherwise
            weight=exp(-u^2); % Normal
    end
    
    CDF_diff=CDF_diff+weight*(CDF1y(n)-CDF2y(n));
end

CDF_diff=CDF_diff/nbBins;
if display>0
    fprintf('\n CDF1 - CDF2 = %f',CDF_diff);
    fprintf('\n On the whole, ');
    if CDF_diff>0
        fprintf('CDF1 seems better than CDF2');
    else
        fprintf('CDF2 seems better than CDF1');
    end
end
end
