function [x,fitness,FEs]=localSearch(x,fitness,FEs,fobj,LB,UB,FEmax,...
    quantis,quantOnce, diam, option)
% Here x is the real position, not the normalised one
global isOctave

[N,D]=size(x);
[fCenter,Ind]=min(fitness);
xCenter=x(Ind,:); % Best known position
MaxFunEvals=FEmax-FEs;
if MaxFunEvals<=N return; end

TolFun=1.e-6; TolX=1.e-6;

switch option
    case 0 % Use an Octave or Matlab 'native' search
        
        if isOctave
            ff = @(u)fobj(u');
            options=optimset('MaxFunEvals',MaxFunEvals,'TolX',TolX,'TolFun',TolFun);
            [xNew,fNew,cvg,output] =nonlin_min(ff,xCenter',options);
            
            xNew=xNew';
            FEs=FEs+output.nobjf;
            
        else  % Matlab
            ff = @(u)fobj(u);
            diffEps=0;
            options = optimoptions(@fmincon,'MaxFunEvals',MaxFunEvals,...
                'Algorithm', 'interior-point',...
                'ObjectiveLimit',diffEps,...
                'Display','off','TolX',TolX, 'TolFun',TolFun);
            
            [xNew, fNew,~,output]=...
                fmincon(ff,xCenter,[],[],[],[],LB,UB,[],options);
            FEs=FEs+output.funcCount;
        end
        
        if ~isempty(quantis) && ~quantOnce
            xNew=quantify(xNew,quantis);
            [fNew,FEs]=fEvalAll(xNew,fobj,FEs);
        end
        
        fitness(Ind)=fNew;
        x(Ind,:)=xNew;
        
    case 1 % Pseudo-gradient local search. A rudimentary method
        xTry=xCenter;
        stop=false;
        FEsLocal=0;
        dfBest=0;
        dBest=1;
        stepSign=0;
        step=diam/2; % Rule of thumb ...
        
        while ~stop
            % Find the best dimension to move along
            for d=1:D
                xTry(d)= xTry(d)+TolX;
                fTry=fobj(xTry); FEsLocal=FEsLocal+1;
                df=fCenter-fTry;
                if df>dfBest % Improvement
                    dfBest=df;
                    dBest=d;
                    stepSign=1;
                end
                
                xTry(d)= xTry(d)-2*TolX; % Try the opposite direction
                fTry=fobj(xTry); FEsLocal=FEsLocal+1;
                df=fCenter-fTry;
                if df>dfBest
                    dfBest=df;
                    dBest=d;
                    stepSign=-1;
                end
                xTry(d)=xCenter(d); % Back
            end
            
            % Move
            step=step/2; % Decrease the step size
            if step<TolX
                stop=true;
            else
                xNew=xCenter;
                xNew(dBest)=xNew(dBest)+stepSign*step;
                
                % Evaluate
                fNew=fobj(xNew); FEsLocal=FEsLocal+1;
                dF=abs(fNew-fCenter);
                fitness(Ind)=fNew;
                x(Ind,:)=xNew;
                
                stop=FEsLocal>=N  || dF<=TolFun;              
            end
        end
        FEs=FEs+FEsLocal;
end
end