function [bestBest,bestMedian,bestMean,stdDev, bestBestPos]= ...
    mamsoloop(runNb, pop, FEmax,Problem)

%{
runNb   = number of runs,
pop     = population size
FEmax   = maximum number of evaluations (budget) for each run
Problem = problem to solve (string chain, see getFunc)

bestBest    = best (min) value found over all runs
bestMedian  = mediane of best values of all runs
bestMean    = mean of best values of all runs
bestBestPos = position of the best solution
    
%}
    
    global isOctave
    isOctave = exist('OCTAVE_VERSION', 'builtin') ~= 0;
    if isOctave
        pkg load statistics;
        pkg load optim;
       
      if strncmp(Problem,'CEC2019',7)
      fprintf('\n Sorry, for CEC2019 benchmark you need Matlab \n');
      return
      end   
    end
    
    %----------------------------------------------------  Paths to benchmarks
    global GetFunInfoAddr FuncAddr
    CEC2011Dir = 'CEC_2011_Benchmark';
    addpath(['./', CEC2011Dir]);
    addpath(['./', CEC2011Dir, '/Probs_1_to_8']);
    addpath(['./', CEC2011Dir, '/Prob_9_Transmission_Pricing']);
    addpath(['./', CEC2011Dir, '/Prob_10_Circ_Antenna']);
    addpath(['./', CEC2011Dir, '/Probs_11_ELD_Package/DED Codes']);
    addpath(['./', CEC2011Dir, '/Probs_11_ELD_Package/ELD Codes']);
    addpath(['./', CEC2011Dir,...
        '/Probs_11_ELD_Package/Hydrothermal Codes']);
    addpath(['./', CEC2011Dir, '/Probs_12_to_13_Package']);
    GetFunInfoAddr = @CEC2011get_fun_info;
    FuncAddr = @CEC2011func;
    %------------
    addpath('./CEC_2019_Benchmark');
    addpath('./CEC_2019_Benchmark/input_data');
    
    %---------------------------------------------------- Behaviour parameters
    global allPos allPosSave % Just for a posteriori analysis
    % Should be used only for D=2
    % If Problem='Plateau', it gives a signature
    % of the algorithm:
    %  mamsoloop(10,25,1000,'Plateau');
    allPos=[];
    allPosSave=false;
    
    global verbose
    verbose=false; % In particular to display the solution point
    
    global diamPlot
    diamPlot=false; % WARNING: one plot for each run
    
    fitEvolution=false; %WARNING. Should be used only for one run, or at least
                        %just a few ones
    CDF=false; % Save and plot the cumulative distribution function
   
    
    %---------------------------------------------------------------- Check
    if diamPlot && runNb>5     
      fprintf('\n Are you sure you want to plot %i diameter evolutions?',runNb);
      prompt = ' Y/N : ';
        str = input(prompt,'s');
        if isempty(str)
            str = 'N';
        end
        if str=='N' || str=='n'
            return
        end
        
    end
    
    if fitEvolution && runNb>5
       fprintf('\n Are you sure you want to plot %i fitness evolutions?',runNb);  
        prompt = ' Y/N : ';
        str = input(prompt,'s');
        if isempty(str)
            str = 'N';
        end
        if str=='N' || str=='n'
            return
        end
        
    end
    
    if CDF && runNb<100
      fprintf('\n Are you sure you want to plot the CDF after only %i runs?',runNb); 
      prompt = ' Y/N : ';
        str = input(prompt,'s');
        if isempty(str)
            str = 'N';
        end
        if str=='N' || str=='n'
            return
        end
    end
    %------------------------------------------------------------------ RNG 
    % Initialise the RNG for reproducible results
    if isOctave
        rand('seed',123456789);
        randn('seed',123456789);
        
    else % Matlab
        rng(123456789);
       % rng('shuffle'); % % For unreproducible results
        %             (except just statistically)
    end
    
    % Warm up the RNG
    for t=1:10000	rand;	end
    
    %--------------------------------------------------------- Problem data
    format long
    [LB,UB,D,fobj,quantis,error]=getFunc(Problem);
    if error
        return;
    end
    
    %------------------------------------Prepares files to save the results
  results=createResults('MAMSO'); % Creates the directory for results,
  %                                 with date and time
  global fidResults
  fprintf('\n Results will be saved in %s \n',results);

  fName=[results, 'Results.txt'];
  fidResults=fopen(fName,'w');
  fprintf(fidResults,'Problem  nbRuns Pop. FEmax \n %s %i %i %i \n', Problem,runNb, pop,FEmax);

    if CDF
        fMinSave=fopen([results 'fMinSave'],'w');
        fCDF=fopen([results 'CDF'],'w');
        % You could also save two CDFs and run then CDFcompare
    end 
    %----------------------------------------------------- Loop and results
    for run=1:runNb
       [best(run), bestPos(run,:),fBestEvolution,FEs]=...
         MAMSO(run, pop,FEmax,LB,UB,D,fobj,quantis);
     
        
  % AOA test  (of course, comment the two lines above)
  % Max_iter=floor(FEmax/pop);
 % [bestPos(run,:), best(run),~] = AOA(pop,Max_iter,fobj, D,LB(1),UB(1),1,2) ;     
  %  FEs=FEmax;    
        
        fprintf('\n run %i, %i evaluations => %e',run, FEs, best(run));
        fprintf(fidResults,'\n%i %i %e',run, FEs, best(run));
               
         if fitEvolution %(Warning: one plot by run):
            figure
            plot(fBestEvolution);
            title('Fitness evolution')
            xlabel('Iteration');
            ylabel('Best fitness');
         end
      end % for run=1:runNb

      
   % [bestBest,Ind]=min(best); % BUG sometimes! Very rarely, but ...
                               % Do not always return the right min
                               % but a value greater than the median!
   bestBest=Inf;
   for run=1:runNb
     if best(run)<bestBest
        bestBest=best(run);
        Ind=run;
     end
   end
   
    bestMedian=median(best);
    bestMean=mean(best);
    stdDev=std(best);
    bestBestPos=bestPos(Ind,:);
    D=length(bestBestPos);
    
 
    fprintf('\n Function %s, dimension %i',Problem, D)
    fprintf('\n Population %i, %i evaluations', pop, FEs)
    fprintf('\n %i runs',runNb);
    fprintf('\n min   median   mean \n')
    fprintf(' %e \n %e \n %e',bestBest, bestMedian, bestMean)
    
    fprintf(fidResults,'\n Best Median Mean Std_dev \n');
    fprintf(fidResults,'%e %e %e %e',bestBest,bestMedian,bestMean,stdDev);
   
    %if verbose
    fprintf('\n\n Best position:')
    for d=1:D
        fprintf('\n %e', bestBestPos(d))
    end
    %end
    fprintf('\n')
    
    if CDF
        for run=1:runNb
            fprintf(fMinSave,'%e \n',best(run));
        end
        mini=min(best); maxi=max(best); % You can modify them
        nbBins=10;
        option=2;
        name=Problem;
        [CDF,bestHisto,bestHistoCumul,minCDF]=...
            CDFplot (best,mini,maxi,nbBins, option,name);
      [~,nbClass]=size(CDF);
      for n=1:nbClass
        fprintf(fCDF,'%f %f \n',CDF(1,n),CDF(2,n));  
      end
    end
    
    if allPosSave
        if D==1
        fprintf('\n Warning: no 2D plot because dimension is 1 \n'); 
        return
        end
        % Successive positions. Useful to plot a signature
        figure
        scatter(allPos(:,1),allPos(:,2),'filled');
        %set(gca,'xtick',[]);
        %set(gca,'ytick',[]);
        title('All positions');
    end
   
      fprintf('\n Results are saved in %s \n',results);

end